/* C30.H - TMS320C30/C31 INCLUDE FILE */

void c_int99(void);
void init_evm(void);
void init_aic(void);
void init_host(void);
void configure_aic(int i);

#define OFF    0x00
#define ON     0x01

/* TMS320C30 MEMORY-MAPPED CONTROL REGISTER INDICES */
#define GLOBAL 0         /* GLOBAL CONTROL REGISTER */
#define EXPANSION 0      /* EXPANSION BUS */
#define PRIMARY   4      /* PRIMARY BUS */

/* SERIAL PORT CONTROL REGISTERS */
#define X_PORT  2        /* TRANSMIT CONTROL */
#define R_PORT  3        /* RECEIVE  CONTROL */
#define X_DATA  8        /* TRANSMIT DATA */
#define R_DATA 12        /* RECEIVE  DATA */

#define PERIOD  8        /* TIMER PERIOD REGISTER */

/* DMA CONTROL REGISTERS */
#define SOURCE   4       /* SOURCE ADDRESS REGISTER */
#define DEST     6       /* DESTINATION ADDRESS REGISTER */
#define TRANSFER 8       /* TRANSFER COUNTER REGISTER */

/* AIC VOLTAGE INPUT CONTROL */
#define THREE_V    1
#define LINE_V     2

/* AIC COMMAND WORD BITFIELD ENCODING STRUCTURES */
typedef struct
{
    unsigned int command :2;     /* COMMAND BITS */
    unsigned int ra      :5;     /* RECEIVE COUNTER A LOAD VALUE */
    unsigned int d_78    :2;     /* UNUSED */
    unsigned int ta      :5;     /* TRANSMIT COUNTER A LOAD VALUE */
    unsigned int d_ef    :2;     /* UNUSED */
} AIC_COMMAND_0;

typedef struct
{
    unsigned int command  :2;    /* COMMAND BITS --- SHOULD BE SET TO 01*/
      signed int ra_prime :6;    /* RECEIVE COUNTER DELTA A' LOAD VALUE */
    unsigned int d_8      :1;    /* UNUSED */
      signed int ta_prime :6;    /* RECEIVE COUNTER DELTA A' LOAD VALUE */
    unsigned int d_f      :1;    /* UNUSED */
} AIC_COMMAND_1;

typedef struct
{
    unsigned int command :2;     /* COMMAND BITS --- SHOULD BE SET TO 10*/
    unsigned int rb      :6;     /* RECEIVE COUNTER B LOAD VALUE */
    unsigned int d_8     :1;     /* UNUSED */
    unsigned int tb      :6;     /* TRANSMIT COUNTER B LOAD VALUE */
    unsigned int d_f     :1;     /* UNUSED */
} AIC_COMMAND_2;

typedef struct
{
    unsigned int command  :2;    /* COMMAND BITS --- SHOULD BE SET TO 11*/
    unsigned int highpass :1;    /* HIGHPASS FILTER ENABLE */
    unsigned int loopback :1;    /* LOOPBACK TEST ENABLE */
    unsigned int aux      :1;    /* AUX INPUT ENABLE */
    unsigned int sync     :1;    /* SYNCHRONOUS TRANSMIT/RECEIVE ENABLE */
    unsigned int gain     :2;    /* GAIN SELECTION BITS */
    unsigned int d_8      :1;    /* UNUSED */
    unsigned int sinx     :1;    /* SINX/X CORRECTION FILTER ENABLE */
    unsigned int d_abcdef :1;    /* UNUSED */
} AIC_COMMAND_3;
