#include <math.h>
#include <libap.h>
#include "rtdspc.h"
#include "filter.h"

/***********************************************************************

MKGWN.C - Gaussian Noise Filter Example for DSP32

This program filters a sine wave with added Gaussian noise.  It
implements a 35 point FIR filter (stored in variable fir_lpf35)
on an generated signal.  The filter is a LPF with 40 dB out of
band rejection.  The 3 dB point is at a relative frequency of
approximately .25*fs.

************************************************************************/

float sigma = 0.2;
int bypass = 0;
float hist[34];
float sinvals[128];

void main()
{
  register int          i;
  register float x,y;

  for(i = 0 ; i < 128 ; i++)
      sinvals[i]=sin(0.0625*PI*(float)(i));

  for(i = 0 ; ; i++) {
      x = sinvals[i&127] + sigma*gran();
      x *= 2000.0;         /* scale for D/A converter */
      y = fir_filter(x,fir_lpf35,35,hist);
      if(!bypass)
        sendout(y);
      else
        sendout(hist[16]);  /* compensate for delay of filter */
/* change key a little bit every 32K samples */
      if(i == 0x8000) {
        bypass ^= 1;
        i = 0;
      }
  }
}
